#!/bin/sh
#
# Script to create a Mac OS X disk image for mpgedit
#
pmaker=/Developer/Applications/Utilities/PackageMaker.app/Contents/MacOS/PackageMaker
if [ `id -u -n` != "admin" ]; then
  echo "This script must be run as admin"
  exit 1
fi

# dist_parse_version.sh sets "RELEASE" from version.h
. `dirname $0`/dist_parse_version.sh $0

(
  if [ ! -f NEWS.sedit ]; then
    ./mk_install_resources_readme.sh
  fi
  if [ ! -f README.sedit ]; then
    ./mk_install_resources_license.sh
  fi
  cd macosx_installer
  if [ ! -d mpgedit ]; then
    ./maketree.sh
  fi
  if [ ! -d MPGEDIT.pkg ]; then
    echo $pmaker -build \
    -p `pwd`/MPGEDIT.pkg \
    -f `pwd`/mpgedit \
    -r `pwd`/Install_Resources \
    -i `pwd`/etc/Info.plist \
    -d `pwd`/etc/Description.plist

    $pmaker -build \
    -p `pwd`/MPGEDIT.pkg \
    -f `pwd`/mpgedit \
    -r `pwd`/Install_Resources \
    -i `pwd`/etc/Info.plist \
    -d `pwd`/etc/Description.plist
  fi
)

rm -f /tmp/mpgedit_work.dmg /tmp/mpgedit.dmg

hdiutil create -megabytes 20 /tmp/mpgedit_work.dmg -layout NONE
vol=`hdid -nomount /tmp/mpgedit_work.dmg`
echo "debug vol=$vol"

cat /etc/sudoadminpwd | sudo -S -u root newfs_hfs -v mpgeditinstall $vol

echo "ejecting disk $vol"
hdiutil eject $vol

echo "mounting disk '/tmp/mpgedit_work.dmg'"
tmp=`hdid /tmp/mpgedit_work.dmg`
target=`echo $tmp | awk '{print $2}'`
echo debug "mounted as '$target'"

echo "Copying 'macosx_installer/MPGEDIT.pkg' to '$target'"
cp -pr macosx_installer/MPGEDIT.pkg $target

hdiutil eject $vol

hdiutil convert -format UDZO /tmp/mpgedit_work.dmg -o /tmp/mpgedit.dmg

rm -f mpgedit.dmg

# Copy built disk image name with release version number
#
DESTDMGNAME="mpgedit_${RELEASE}.dmg"
cp /tmp/mpgedit.dmg $DESTDMGNAME
ls -l /tmp/mpgedit.dmg
ls -l $DESTDMGNAME
rm -f /tmp/mpgedit.dmg
[ -s ../../$DESTDMGNAME ] || ln -s `pwd`/$DESTDMGNAME ../../

# Following link satisfies makefile dependency
[ -s mpgedit.dmg ] || ln -s $DESTDMGNAME mpgedit.dmg
