#!/bin/sh
#
# Script to create a Mac OS X disk image for xmpgedit
#
if [ `id -u -n` != "admin" ]; then
  echo "This script must be run as admin"
  exit 1
fi


. `dirname $0`/dist_parse_version.sh $0

cd gui
rm -f /tmp/xmpgedit_gtk_work.dmg /tmp/xmpgedit_gtk.dmg


#
# Populate the xmpgedit.app/Contents/Resources directory
#
xpm_files="\
blankdigit_led.xpm      colon_led.xpm           five_led.xpm
one_led.xpm             play.xpm                seven_led.xpm
two_led.xpm             blankpunct_led.xpm      colon_minute_led.xpm
four_led.xpm            pause.xpm               record.xpm
six_led.xpm             volume1.xpm             close.xpm
eight_led.xpm           next_t.xpm              period_led.xpm
record_green.xpm        stop.xpm                zero_led.xpm
colon_hour_led.xpm      eject.xpm               nine_led.xpm
period_second_led.xpm   record_red.xpm          three_led.xpm"

dylib_files="\
../libdecoder_mad.dylib         ../libmpgedit.dylib
../libdecoder_mpg123.dylib      ../libmpgedit_decoder.dylib
../libdecoder_popen.dylib"

macos_files="\
xmpgedit.app/Contents/MacOS/xmpgedit_gtk.c
xmpgedit.app/Contents/MacOS/xmpgedit_gtk.sh"

lib_files="\
./libgtk-x11-2.0.0.400.9.dylib
./libgtk-x11-2.0.dylib
./libgtk-x11-2.0.0.dylib
./libgdk-x11-2.0.0.400.9.dylib
./libgdk-x11-2.0.0.dylib
./libgdk-x11-2.0.dylib
./libatk-1.0.0.600.1.dylib
./libatk-1.0.0.dylib
./libatk-1.0.dylib
./libgdk_pixbuf-2.0.0.400.9.dylib
./libgdk_pixbuf-2.0.0.dylib
./libgdk_pixbuf-2.0.dylib
./libpangoxft-1.0.0.600.0.dylib
./libpangoxft-1.0.0.dylib
./libpangoxft-1.0.dylib
./libpangox-1.0.0.600.0.dylib
./libpangox-1.0.0.dylib
./libpangox-1.0.dylib
./libpango-1.0.0.600.0.dylib
./libpango-1.0.0.dylib
./libpango-1.0.dylib
./libgobject-2.0.0.400.6.dylib
./libgobject-2.0.0.dylib
./libgobject-2.0.dylib
./libgmodule-2.0.0.400.6.dylib
./libgmodule-2.0.0.dylib
./libgmodule-2.0.dylib
./libglib-2.0.0.400.6.dylib
./libglib-2.0.0.dylib
./libglib-2.0.dylib
./libintl.3.4.3.dylib
./libintl.3.dylib
./libintl.dylib
./libiconv.2.3.0.dylib
./libiconv.2.dylib
./libiconv.dylib
./libintl.1.0.1.dylib
./libintl.1.dylib
./libpangoft2-1.0.0.600.0.dylib
./libpangoft2-1.0.0.dylib
./libpangoft2-1.0.dylib
./gtk-2.0
./gtk-2.0/2.4.0
./gtk-2.0/2.4.0/loaders
./gtk-2.0/2.4.0/loaders/libpixbufloader-ani.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-xbm.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-wbmp.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-tiff.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-tga.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-ras.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-pnm.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-png.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-pcx.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-jpeg.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-ico.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-gif.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-bmp.so
./gtk-2.0/2.4.0/loaders/libpixbufloader-xpm.so
pango/1.4.0/modules/pango-hangul-fc.so
pango/1.4.0/modules/pango-indic-fc.so
pango/1.4.0/modules/pango-hebrew-fc.so
pango/1.4.0/modules/pango-thai-fc.so
pango/1.4.0/modules/pango-basic-x.so
pango/1.4.0/modules/pango-basic-fc.so
pango/1.4.0/modules/pango-arabic-fc.so"

etc_files="\
pangorc
pango.modules
pangox.aliases
gdk-pixbuf.loaders"

info_plist="xmpgedit.app/Contents/Info.plist"

mkdir -p /tmp/xmpgedit.app/Contents/Resources
mkdir -p /tmp/xmpgedit.app/Contents/MacOS
mkdir -p /tmp/xmpgedit.app/Contents/MacOS/etc
mkdir -p /tmp/xmpgedit.app/Contents/MacOS/lib

cp -f $xpm_files   /tmp/xmpgedit.app/Contents/Resources
cp -f $dylib_files /tmp/xmpgedit.app/Contents/Resources
cp -f $macos_files /tmp/xmpgedit.app/Contents/MacOS
cp -f $info_plist  /tmp/xmpgedit.app/Contents
cp -f xmpgedit     /tmp/xmpgedit.app/Contents/Resources
(cd xmpgedit.app/Contents/MacOS/lib && find $lib_files | cpio -pd /tmp/xmpgedit.app/Contents/MacOS/lib)
(cd ../../../src/mpgedit/gui/xmpgedit.app/Contents/MacOS/etc && find $etc_files | cpio -pd /tmp/xmpgedit.app/Contents/MacOS/etc)

(cd /tmp/xmpgedit.app/Contents/MacOS
 cc -o xmpgedit xmpgedit_gtk.c
)

hdiutil create -megabytes 40 /tmp/xmpgedit_gtk_work.dmg -layout NONE
vol=`hdid -nomount /tmp/xmpgedit_gtk_work.dmg`
echo "debug vol=$vol"

cat /etc/sudoadminpwd | sudo -S -u root newfs_hfs -v xmpgeditinstall $vol

echo "ejecting disk $vol"
hdiutil eject $vol

echo "mounting disk '/tmp/xmpgedit_gtk_work.dmg'"
tmp=`hdid /tmp/xmpgedit_gtk_work.dmg`
target=`echo $tmp | awk '{print $2}'`
echo debug "mounted as '$target'"

echo "Copying xmpgedit.app to '$target'"
cp -pr /tmp/xmpgedit.app $target

hdiutil eject $vol

hdiutil convert -format UDZO /tmp/xmpgedit_gtk_work.dmg -o /tmp/xmpgedit_gtk.dmg

cd ..
rm -f xmpgedit_gtk.dmg
DESTDMGNAME="xmpgedit_gtk_${RELEASE}.dmg"

cp /tmp/xmpgedit_gtk.dmg $DESTDMGNAME
(cd /tmp && rm -rf /tmp/xmpgedit.app)
ls -l $DESTDMGNAME
ls -l /tmp/xmpgedit_gtk.dmg
rm -f /tmp/xmpgedit_gtk.dmg
[ -s ../../$DESTDMGNAME ] || ln -s `pwd`/$DESTDMGNAME ../..

# Following link satisfies makefile dependency
[ -s xmpgedit_gtk.dmg ] || ln -s $DESTDMGNAME xmpgedit_gtk.dmg
